<?php
/**
 * SqLinguae pour PHP 5+
 *  (nécessite PHP5+ avec ses modules PDO et SQLITE disponibles sur le serveur Web)
 * 
 * @version 2.1
 * @since 22/01/2011 
 * @author Billig
 * @link http://linguae.stalikez.info
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCILL (GNU-GPL compatible) 
 * @package Script_SqLinguae
 */


include('config.php'); // charge les paramètres de configuration


// Chemin du script relativement à la page-hôte du visualiseur
// éventuellement défini dans la page-hôte, sinon par défaut
$sqlng_conf['scriptdir'] = isset($sqlng_scriptdir) ? $sqlng_scriptdir : 'sqlinguae/';



sqlng_insert();          // lance le script



/***************************************************************************//**
 * Cette fonction encapsule le script proprement dit, elle est appelée
 * automatiquement par l'inclusion de ce fichier dans la page-hôte.
 * L'interface et les éventuels résultats de recherche sont affichés. 
 * 
 * @return none 
 */
function sqlng_insert()
    {
    
    $sqlng_version = '2.1';
    
    global $sqlng_conf; // Array des paramètres de configuration
    global $sqlng_str;  // Array des chaînes de langue
    
    include('statupdate.php'); // fonction de mise à jour des statistiques
    
    
    // ----------------------------------------------------------
    // --- Contrôles de compatibilité du serveur avec ce script  
    // ----------------------------------------------------------
    
    $t = PHP_VERSION;
    if ($t[0] < 5)
        {
        // Message : "Ce script nécessite PHP5"
        die('<p class="warn"><br />' . $sqlng_str[1] . PHP_VERSION . '</p>' );
        }
        
    if (!phpversion('pdo'))
        {
        // Message : "Ce script nécessite qur PDO soit activé"
        die('<p class="warn"><br /><b>' . $sqlng_str[2] . '</b></p>' );
        }
    
    
    // ---------------------------------------------------------
    // --- Récupération des paramètres d'interrogation du dico  
    // ---------------------------------------------------------

    $showpr = (isset($_POST['sqlng_showpr_x'])) ? True : False;   // Afficher les propriétés
    $showexp = (isset($_POST['sqlng_showexp_x'])) ? True : False; // Afficher l'Explorateur
    
    // Motif (texte) à rechercher : passe en POST sauf pour les liens directs du
    // type {{mot}} ou il passe en GET (url))
    $stxt =(isset($_REQUEST['sqlng_stxt'])) ?  htmlspecialchars(trim($_REQUEST['sqlng_stxt'])) : '';
    
    // Les autres paramètres passent en POST pour l'interrogation standard mais
    // passent en GET dans les liens des relations des entrées (racines,
    // synonymes, etc.) ou pour le changement de langue d'interface
    $fdic =(isset($_REQUEST['sqlng_fdic'])) ? htmlspecialchars($_REQUEST['sqlng_fdic']) : '';
    $wid =(isset($_REQUEST['sqlng_wid'])) ? htmlspecialchars($_REQUEST['sqlng_wid']) : '';
    $dom =(isset($_REQUEST['sqlng_dom'])) ? htmlspecialchars($_REQUEST['sqlng_dom']) : 'entr';
    $selkb = (isset($_REQUEST['sqlng_selkb'])) ? htmlspecialchars($_REQUEST['sqlng_selkb']) : ''; 
    $lg = (isset($_REQUEST['sqlng_lg'])) ? htmlspecialchars($_REQUEST['sqlng_lg']) : $sqlng_conf['def_lang']; 



    // -----------------------------------------------------------------------------
    // --- Y-a-til un Dico initial ? (à présélectionner +/- afficher au démarrage)  
    // -----------------------------------------------------------------------------
    
    if (!$fdic && !$stxt && $sqlng_conf['start_dic'])
        {
        if($sqlng_conf['start_dic'][0] == '+') // avec explorateur
            {
            $fdic = substr($sqlng_conf['start_dic'], 1);
            if(file_exists($sqlng_conf['scriptdir'] . 'dic/' . $fdic))
                {
                $showexp = True;
                }
            else
                {
                $fdic = '';
                }
            }
        else    // sans explorateur
            {
            if(file_exists($sqlng_conf['scriptdir'] . 'dic/' . $sqlng_conf['start_dic']))
                {
                $fdic = $sqlng_conf['start_dic'];   
                }
            }
        }
    
    // ---
    
    // Charger les chaînes de l'interface multilingue 
    // > création de l'Array global $sqlng_str

    include ('lang/' . $lg . '.php' );

    // Préparation du lien pour les relations et les icônes de langue d'interface,
    // on le termine par ? ou &amp; suivant l'url de la page-hôte
    $phpself = htmlspecialchars($_SERVER['PHP_SELF']);
    $phpself .= (strpos($phpself, '?')) ? '&amp;' : '?';  
    
    
    
    // ====================================================
    // === Affichage de l'interface                        
    // ====================================================
    
    
    echo'<div id="sqlng_vw">';   
    
    // --------------------
    
    echo '<form method="post" action="#sqlng_vw" style="margin-top:0px"><fieldset>' . "\n";
    
    // Submit invisible pour éviter que 'Enter' n'appelle l'affichage des propriétés...
    // Nb : Cette combine ne marche pas pour tous les navigateurs (Chrome...)
    echo '<input name="sqlng_goh" style="display:none" type="submit" value="" />';
    
    echo '<table>';
    
    
    // -----------------------
    // --- Liste des dicos    
    // -----------------------
    
    // Etiquette "Dictionnaire :"
    echo '<tr><td>', $sqlng_str[7], '</td><td>';
    
    // Un index des dicos existe > on l'utilise pour construire la liste déroulante
    if (file_exists($sqlng_conf['scriptdir'] . 'dic/dic_idx.php'))
        {
        writeDicLstByIdx($fdic);
        }
    else
    // Index absent ou désactivé > on scanne le répertoire
        {
        writeDicLstByScan($fdic);
        }
    
    // ---------------------------------------------------------
    // --- Créer l'objet base de données du dico sélectionné    
    // ---------------------------------------------------------
    
    if ($fdic)
        {
        $db = new PDO('sqlite:' . $sqlng_conf['scriptdir'] . 'dic/' . $fdic, '', '');   
        }
    
    // ------------------------------------------------
    // --- Bouton-image d'affichage des propriétés     
    // ------------------------------------------------
    
    echo '<input type="image" name="sqlng_showpr" src="', $sqlng_conf['scriptdir'], 'lib/img/propr.gif" value="1" title="', $sqlng_str[34], '" /> &nbsp;';
    
    // --------------------------------------------------------
    // --- Bouton-image d'affichage de l'explorateur du dico   
    // --------------------------------------------------------
    
    echo '<input type="image" style="display:none" name="sqlng_showexp" id="sqlng_showexp" src="', $sqlng_conf['scriptdir'], 'lib/img/idx.gif" value="1" title="', $sqlng_str[71], '" />  &nbsp;';
        
    
    echo '</td></tr>';
    
    // ---------------------------------------------
    // --- Liste de choix du domaine de recherche   
    // ---------------------------------------------
    
    if ($sqlng_conf['lst_dom'])
        {
        echo '<tr><td>', $sqlng_str[50], ' </td><td><select name="sqlng_dom">
        <option value="entr"';
        echo ($dom == 'entr') ? ' selected="selected">' : '>';
        echo  $sqlng_str[51], '</option>
        <option value="not"';
        echo ($dom == 'not') ? ' selected="selected">' : '>';
        echo $sqlng_str[52], '</option>
        </select></td></tr>';
        }
    
    
    // ---------------------------------------------------
    // --- Case de saisie du début de texte à rechercher  
    // ---------------------------------------------------
    
    // infobulle
    $ttl = $sqlng_str[17] . ' (' . $sqlng_conf['char_max'] . ' c. max.) ...  \' % \' ' . $sqlng_str[62];
    
    echo '<tr title="', $ttl, '"><td>', $sqlng_str[8], '</td><td><input name="sqlng_stxt" id="sqlng_stxt" type="text" value="', $stxt, '" maxlength="', $sqlng_conf['char_max'], '" onfocus="sqlng_ctarg=this" />';
       
    
    // ------------------------------------------
    // --- Bouton image d'envoi (> recherche)    
    // ------------------------------------------
    
    echo '
    <input name="sqlng_go" type="image" src="' . $sqlng_conf['scriptdir'] . 'lib/img/goto.gif" style="position:relative;top:3px" /></td></tr>';
    


    // ------------------------------------------------
    // --- Sélecteur de Jeux de caractères spéciaux    
    // ------------------------------------------------
    
    if ($sqlng_conf['lst_char'])
        {
        echo '<tr id="kbselector" title="', $sqlng_str[66], '"><td>', $sqlng_str[64], '</td><td>    
        <select name="sqlng_selkb" onchange="kbchange(this.value,\'', $sqlng_conf['scriptdir'], '\')">
        <option value="">', $sqlng_str[4], '</option>';
    
        foreach ($sqlng_conf['kbs'] as $kb => $t)
            {
            echo ($selkb == $kb) ? '<option value="' . $kb . '" selected="selected">' : '<option value="' . $kb  .'">';
            echo $t, '</option>';
            }
        
        echo '</select></td></tr>';
        }
    
    // --------------------------------
    // --- Fermeture du formulaire     
    // --------------------------------

    echo'</table>
    </fieldset></form>';
    
    // On rend visible par Javascript des éléments masqués par CSS, ceux-ci resteront
    // donc masqués si Javascript est désactivé
    // 1) le sélecteur  de jeux de caractères (Utilisation de 'visibility' plutôt
    //    que 'display' car réaffichage incorrect avec 'display'
    // 2) le bouton d'affichage de l'Explorateur
    if ($sqlng_conf['lst_char'])
        {
        echo "\n", '<script type="text/javascript">
        document.getElementById("kbselector").style.visibility="visible";
        document.getElementById("sqlng_showexp").style.display="inline";
        </script>', "\n";
        }
    
    // ---------------------------------------------------------------------------
    // --- Bloc des touches du Clavier (ce bloc sera rempli par une requête Ajax) 
    // ---------------------------------------------------------------------------
    
    echo '<div id="sqlng_vkb"></div>';
    
    // Moteur Javascript du sélecteur de clavier et du clavier lui-même
    echo "\n", '<script type="text/javascript" src="', $sqlng_conf['scriptdir'], 'lib/kbengine.js"></script>', "\n";

    
    if ($selkb)
        {
        // réafficher le bloc de touches
        echo "\n", '<script type="text/javascript">kbchange(\'', $selkb , '\',\'', $sqlng_conf['scriptdir'], '\');</script>', "\n";
        }
    
    
    // --------------------------------------------------------------
    // --- Préparation de la chaîne des liens-images des greffons    
    // --------------------------------------------------------------
    
    $htmplug = ($fdic) ? getHtmOfBtPlugins($db) : '';
    
    
    
    // =========================================================================
    // === Aiguillage : Recherche / Propriétés / Explorateur                    
    // =========================================================================
    
    
    if ($showpr) // --- Affichage des propriétés
        {
        if (!$fdic)
            {
            // Message : "Choisissez un dictionnaire"
            echo '<p class="warn">', $sqlng_str[18], '</p>';
            }
        else
            {
            echo getHtmOfProperties($fdic);        
            }
        }
    
    elseif ($showexp) // --- Affichage de l'explorateur du dico
        {
        if (!$fdic)
            {
            // Message : "Choisissez un dictionnaire"
            echo '<p class="warn">', $sqlng_str[18], '</p>';
            }
        else
            {
            echo getHtmOfExplorer($fdic, $from=0, $lg, $phpself, $htmplug);
            }
        }
    else // --- Recherche directe
    {
    
    include('getword.php'); // Contient la fonction de mise en forme html d'un mot
    

    // ===============================================================
    // === Recherche dans la base-dico et affichage des résultats     
    // ===============================================================
    
    // Préparation de la liste des libellés des relations (pour éviter de la
    // redéfinir à chaque itération de la boucle d'affichage)
    $lstrel = Array (   'roots' => $sqlng_str[9],
                        'syn'   => $sqlng_str[10],
                        'see'   => $sqlng_str[11],
                        'ant'   => $sqlng_str[12]);

    
    // --------------------------------------------------------------
    // --- Contrôles et formatages préalables du motif de recherche  
    // --------------------------------------------------------------

    if(trim($stxt) == '%')
        {
        $stxt = '';
        echo '<blockquote><p class="warn"><b> . . . ? ? ? . . .</b></p></blockquote>';
        }

    
    // ----------------------------------------------------------------
    if ($stxt && $fdic) // --- Rechercher à partir du formulaire       
    // ----------------------------------------------------------------
        {
        
        $success = writeResForPattern($fdic, $dom, $stxt, $lstrel, $phpself, $lg, $htmplug);
        
        
        // Compléter les statistiques de consultation du dictionnaire (optionnel) 
        if ($sqlng_conf['record_stat'] & 2)
            {
            statUpdate($stxt, $fdic, $success, $scriptdir = $sqlng_conf['scriptdir'] );
            }
        
        }
    // ---------------------------------------------------------------------
    elseif ($wid) // Rechercher à partir d'un wordID (liens de relations >) 
    // ---------------------------------------------------------------------
        {
        
        // Connexion  à la base-dico sqlite
        $db = new PDO('sqlite:' . $sqlng_conf['scriptdir'] . 'dic/' . $fdic, '', '');    
        
        $res = $db->query('SELECT * FROM Data WHERE wordid = ' . $db->quote($wid));    
        $res = $res->fetch(PDO::FETCH_ASSOC);
        
        echo getHtmOfWordData($res, $db, $lstrel, $fdic, $phpself, $dom, $stxt, $srl=False, $lg, $htmplug);
        
        }
    // -------------------------------------------------------------------
    else // --- Données incomplètes > avertir                             
    // -------------------------------------------------------------------
        {
        if ((!$stxt && !$fdic)||(!$stxt && $fdic))
            {
            if(isset($_POST['sqlng_go_x'])||isset($_POST['sqlng_goh']))
                {
                // Message : "Tapez le début du mot..."
                echo '<p class="warn">', $sqlng_str[17], ' (', $sqlng_conf['char_max'], ' c. max.) ...</p><p class="warn">\' % \' ', $sqlng_str[62], '</p>';
                }
            }
        else
            {
            // Message : "Choisissez un dictionnaire"
            echo '<p class="warn">', $sqlng_str[18], '</p>';
            }
            
        }
    
    
    }// fin aiguillage
    
    
    // --------------------------------------------
    // -- Fin de l'affichage du visualiseur        
    // --------------------------------------------
    
    
    // --- icônes de changement de langue d'interface   
    
    echo "\n", '<p class="toscript">';
    
    if (count($sqlng_conf['ic_lang']) > 1)
        {
        foreach($sqlng_conf['ic_lang'] as $ic)
            {
            if (file_exists($sqlng_conf['scriptdir'] . 'lib/lang/' . $ic . '.gif' ))
                {
                echo '<a href="' . $phpself . 'sqlng_lg=' . $ic . '"><img src="', $sqlng_conf['scriptdir'], 'lib/lang/', $ic, '.gif" alt="', $ic, '" /></a> ';
                }
            else
                {
                echo '<a href="' . $phpself . 'sqlng_lg=' . $ic . '"><img src="', $sqlng_conf['scriptdir'], 'lib/lang/any.gif" alt="', $ic, '" /></a> ';
                }
            }
        }
    
    // --- Lien d'info du script : "Animé par..."
    echo $sqlng_str[19], ' <b><a href="http://linguae.stalikez.info">SqLinguae</a> ', $sqlng_version, '</b>, ', $sqlng_str[20], '</p>';
    
    
    // Fermeture du bloc d'affichage du script dans la page-hôte
    echo "\n</div>\n";
    
    
    
    
    }// fin function sqlng_insert()



/***************************************************************************//**
 * Retourne l'Html d'affichage de l'explorateur de dictionnaire
 *
 * @param string $fdic Nom du fichier du dictionnaire courant
 * @param integer $from Numéro d'index du premier élément de la liste à afficher
 * @param string $lg Langue d'interface  
 * @param string $phpself Url des liens de relations (terminé par '?' ou '&amp;') 
 * @param string $htmplug Chaîne des liens de plugins du dico 
 * @return string Le code html à afficher 
 */
 function getHtmOfExplorer($fdic, $from, $lg, $phpself, $htmplug)
    {
    
    global $sqlng_conf; // Array des paramètres de configuration
    global $sqlng_str;  // Array des chaînes de langue
    
    // Connexion  à la base-dico sqlite pour extraire le nb d'entrées
    $db = new PDO('sqlite:' . $sqlng_conf['scriptdir'] . 'dic/' . $fdic, '', '');        
    $res = $db->query('SELECT wordcount FROM Properties');   
    $res = $res->fetch(PDO::FETCH_ASSOC);   
    $wordcount = $res['wordcount'];
    $db = null;
    
    $htm = '';
    
    // Moteur Javascript de l'explorateur de dico
    $htm .= "\n" . '<script type="text/javascript" src="' . $sqlng_conf['scriptdir'] . 'lib/idxengine.js"></script>' . "\n";    
       
    
    // Conteneur général de l'Explorateur (<table> de 2 cases)
    $htm .= '<table id="sqlng_expl"><tr><td style="vertical-align:top;width:' . $sqlng_conf['expl_width'] . 'px">';
       
    // Conteneur de la liste, de sa case Gotow et de sa barre de navigation
    $htm .= '<table style="width:' . $sqlng_conf['expl_width'] . 'px;text-align:left">';
    
    // Case Gotow et bouton
    $ttl = $sqlng_str[17] . ' (' . $sqlng_conf['char_max'] . ' c. max.) ...';
    $htm .= '<tr><td title="' . $ttl . '">' . $sqlng_str[72] .
    ' <input id="sqlng_gotow" type="text" maxlength="' . $sqlng_conf['char_max'] . '" onkeypress="entergo(event)" onfocus="sqlng_ctarg=this" />&nbsp;<a href="javascript:gotow()"><img src="' . $sqlng_conf['scriptdir'] . 'lib/img/goto.gif" alt="" style="position:relative;top:3px" /></a></td><td>&nbsp;</td></tr>';
    
    // Liste des entrées du dico (div nécessaire car nowrap ne marche pas sur un td avec IE)
    $htm .= '<tr><td><div id="sqlng_idx" style="width:' . $sqlng_conf['expl_width'] . 'px"></div></td>';
    
    // Barre de navigation dans l'explorateur
    $htm .= '<td  style="background:#DADADA;width:15px;padding:0px">
    <table id="sqlng_nav"><tr><td style="vertical-align:top;padding:0px">
    <p><a href="javascript:upall()"><img src="' . $sqlng_conf['scriptdir'] . 'lib/img/upall.gif" alt="" /></a></p>
    <p><a href="javascript:upfst()"><img src="' . $sqlng_conf['scriptdir'] . 'lib/img/upfst.gif" alt="" /></a></p>
    <p><a href="javascript:upsz()"><img src="' . $sqlng_conf['scriptdir'] . 'lib/img/upsz.gif" alt="" /></a></p>
    <p><a href="javascript:up1()"><img src="' . $sqlng_conf['scriptdir'] . 'lib/img/up1.gif" alt="" /></a><br /></p>
    </td></tr><tr><td style="vertical-align:bottom;padding:0px">
    <p><a href="javascript:dwn1()"><img src="' . $sqlng_conf['scriptdir'] . 'lib/img/dwn1.gif" alt="" /></a></p>
    <p><a href="javascript:dwnsz()"><img src="' . $sqlng_conf['scriptdir'] . 'lib/img/dwnsz.gif" alt="" /></a></p>
    <p><a href="javascript:dwnfst()"><img src="' . $sqlng_conf['scriptdir'] . 'lib/img/dwnfst.gif" alt="" /></a></p>
    <p><a href="javascript:dwnall()"><img src="' . $sqlng_conf['scriptdir'] . 'lib/img/dwnall.gif" alt="" /></a></p>
    </td></tr>
    </table>
    </td></tr></table></td>';
    
    // Case d'affichage de l'index numérique du mot (sera remplie par une requête ajax)
    $htm .= '<td style="width:100%;vertical-align:bottom;padding-bottom:2px"><div id="sqlng_num">...</div>';
    
    // Case d'affichage des données du mot (sera remplie par une requête ajax)
    $htm .= '<div id="wddata"></div></td></tr></table>';
    
    // Affichage initial (lancer un script JS qui affiche l'explorateur et initialise un tableau de nombreuses variables JS)
    $recstat = ($sqlng_conf['record_stat'] & 4) ? '1' : '';
    $safehtmplug = str_replace('<', '[)1(]', $htmplug);
    $safehtmplug = str_replace('>', '[)2(]', $safehtmplug);
    $htm .= "\n" . '<script type="text/javascript">
            <!--
            show1(\''  . $fdic
             . '\',\'' . $sqlng_conf['expl_nbl']
             . '\',\'' . $sqlng_conf['scriptdir']
             . '\',\'' . $lg
             . '\',\'' . $phpself
             . '\',\'' . $wordcount
             . '\',\'' . $recstat
             . '\',\'' . $safehtmplug
             . '\');
             //-->
             </script>' . "\n";  
    
    return $htm;
    }




/***************************************************************************//**
 * Ecrit la liste des dicos disponibles en scannant le répertoire des dicos
 * 
 * @param string $fdic Nom du fichier du dictionnaire courant
 * @return none
 */
function writeDicLstByScan ($fdic)
    {
    
    global $sqlng_conf; // Array des paramètres de configuration
    global $sqlng_str;  // Array des chaînes de langue
    
    // Lister le répertoire des dicos
    $dicos = Array();
    $repdic = opendir($sqlng_conf['scriptdir'] . 'dic');

    while($f = readdir($repdic))
        {
    	if($f[0] != '.') // élimine du listage '.', '..', '.htaccess' et '.dic_idx.php'
            {
    		$dicos[] = $f;
            }
    	}
    closedir($repdic);
    
    // ---  
        
    if (count($dicos) == 0)
        {
        // Message : "Aucun dico n'est installé"
        die('<p class="warn">' . $sqlng_str[3] . '</p>');
        
        }
    elseif (count($dicos) > 1)
        {
        // Si le répertoire des dicos contient plus d'un dico, une liste déroulante
        // des noms conviviaux des dicos est affichée
    
        echo "\n", '<select name="sqlng_fdic" onchange="this.form.submit()">
        <option value="">', $sqlng_str[4], '</option>', "\n"; // (choisir...)
        
        foreach($dicos as $dico)
            {
            echo '<option value="', $dico, '"';
            echo ($dico == $fdic) ? ' selected="selected">' : '>';
            echo getDicName($sqlng_conf['scriptdir'] . 'dic/' . $dico), "</option>\n";
            }
        
        echo "</select>\n";
        
        }
    else    // dico unique > simple affichage de son nom convivial
        {        
        echo '<b>', getDicName($sqlng_conf['scriptdir'] . 'dic/' . $dicos[0]), '</b>
        <input type="hidden" name="sqlng_fdic" value="', $dicos[0], '" />';
        }
    
    }




/***************************************************************************//**
 * Retourne le nom convivial du dico dont le chemin du fichier est passé en
 * argument. Si ce dico ne contient pas un tel nom, le nom du fichier sans son
 * extension est retourné.
 *
 * @param $filepath string Chemin absolu ou relatif du fichier-dico 
 * @return string Nom convivial du dico
 */
function getDicName($fpath)
    {
    
    global $sqlng_str;  // Array des chaînes de langue
    
    // Connexion  à la base-dico sqlite pour en extraire le nom convivial
    $db = new PDO('sqlite:' . $fpath, '', '');        
    $res = $db->query('SELECT dicName FROM Properties');   
            
    if ($res)
        {
        $res = $res->fetch(PDO::FETCH_ASSOC);
        $nom = $res['dicName'];
        
        if ($nom)
            {
            $nom = htmlspecialchars($nom);
            }
        else
            {
            // si le nom est absent, on utilise le nom de fichier sans extension  
            $nom = basename($fpath);
            $nom = ucWords(substr($nom, 0, strrpos($nom, '.')));
            }
        }
    else
        {
        // Permettre l'affichage du message d'erreur si la fonction est appelée par
        // une boucle de remplissage d'un <select>
        echo '</option></select>';
         
        // Message : "le fichier xxx n'est pas un dico SqLinguae valide"
        die('<p class="warn">' . $sqlng_str[5] . ' \'' . $fpath . '\' ' . $sqlng_str[6] . '</p>');
        }
        
    $db = null;
    
    return $nom;
    
    }



/***************************************************************************//**
 * Ecrit la liste des dicos disponibles en utilisant le fichier index
 * 
 * @param string $fdic Nom du fichier du dictionnaire courant
 * @return none
 */
function writeDicLstByIdx ($fdic)
    {
    
    global $sqlng_conf; // Array des paramètres de configuration
    global $sqlng_str;  // Array des chaînes de langue

    // Charger l'index des dicos
    include ($sqlng_conf['scriptdir'] . 'dic/dic_idx.php');
    
           
    
    echo "\n", '<select name="sqlng_fdic">
        <option value="">', $sqlng_str[4], '</option>', "\n"; // (choisir...)
    
    $flaggroup = false;
    
    foreach ($dicos as $dico)
        {
        $dico = trim($dico);
        if ($dico[0] == '#')
            {
            echo ($flaggroup) ? '</optgroup>' : '';
            echo '<optgroup label="', trim(substr($dico, 1)), '">';
            $flaggroup = true;
            }
        else
            {
            $d = explode('|', $dico); // $d[0] : texte, $d[1] : fichier 
            
            $f = trim($d[1]);
            
            if (!file_exists($sqlng_conf['scriptdir'] . 'dic/' . $f))
                {
                die('</select><p class="warn>' . $sqlng_str[53] . ' \'' . $f . '\'</p>');
                }
            
            echo '<option value="', $f, '"';
            echo ($f == $fdic) ? ' selected="selected">' : '>';
            echo trim($d[0]), "</option>\n";
            }
        }
    
    if ($flaggroup)
        {
        echo '</optgroup></select>', "\n";
        }
    else
        {
        echo "</select>\n";
        }
    
    }




/***************************************************************************//**
 * Ecrit l'html des résultats de la recherche d'un motif dans un dictionnaire
 * 
 * @param string $fdic Nom du fichier du dictionnaire dans lequel chercher
 * @param string $dom Domaine de recherche dans le dico (entrées, notices...)
 * @param string $stxt Motif recherché
 * @param Array $lstrel Array associatif des libellés des relations
 * @param string $phpself Url des liens de relations (terminé par '?' ou '&amp;')
 * @param string $lg Langue d'interface  
 * @param string $htmplug Chaîne des liens de plugins du dico 
 * @return integer (0|1|2) Succès de la recherche, 0: échec 1: direct 2: approchant
 */
function writeResForPattern($fdic, $dom, $stxt, $lstrel, $phpself, $lg, $htmplug)
    {
    
    
    global $sqlng_conf; // Array des paramètres de configuration
    global $sqlng_str;  // Array des chaînes de langue
    
    
    // ==============================================
    // === Rechercher le motif dans le dico          
    // ==============================================
        
    // Connexion  à la base-dico sqlite
    $db = new PDO('sqlite:' . $sqlng_conf['scriptdir'] . 'dic/' . $fdic, '', '');    
        
    if ($dom == 'entr') // domaine de recherche : entrées
            {
            
            // Compter les résultats sans limitation
            $res = $db->query('SELECT COUNT(wordid) FROM Data
                               WHERE word LIKE ' . $db->quote($stxt . '%'));   
            $nbtot_res = $res->fetch();
            $nbtot_res = $nbtot_res[0];
            
            // Extraire avec limitation optionnelle
            $res = $db->query('SELECT * FROM Data
                               WHERE word LIKE ' . $db->quote($stxt . '%') . '
                               LIMIT ' . $sqlng_conf['res_max'] );   
            }
    else // domaine de recherche : notices
            {
            
            // Compter les résultats sans limitation
            $res = $db->query('SELECT COUNT(wordid) FROM Data
                               WHERE short || long LIKE ' . $db->quote('%' . $stxt . '%')); 
            $nbtot_res = $res->fetch();
            $nbtot_res = $nbtot_res[0];
            
            
            // Extraire avec limitation optionnelle
            $res = $db->query('SELECT * FROM Data
                               WHERE short || long LIKE ' . $db->quote('%' . $stxt . '%') . '
                               LIMIT ' . $sqlng_conf['res_max'] ); 
            }
        
    $res = $res->fetchAll(PDO::FETCH_ASSOC);
    
    
    
    // ==============================================
    // === Traitement des résultats                  
    // ==============================================
    
    $nb_res = count($res); // nombre de résultats réellement affichés
     
    $success = 0;
    
    // --------------------------------------------------------------
    if ($nbtot_res == 0 ) // --- Echec de la recherche initiale        
    // --------------------------------------------------------------
        {
        
        if ($dom == 'entr') // --- Echec de la recherche dans les entrées
            {
            // Message : "pas de correspondance pour xxx dans les entrées"
            // "Réduisez la longueur ou utilisez %"
            echo '<p class="warn">', $sqlng_str[13], '<b>', $stxt, '</b>', $sqlng_str[14], '</p>
                  <p class="warn">', $sqlng_str[57],'</p>';
                
                
            // Faire automatiquement une recherche raccourcie dans les entrées
            // (avec échec silencieux) de -1 ou -2 caractères
            // si le motif a 3 caractères minimum
            if (strlen($stxt) > 2)
                {
                $stxt = substr($stxt, 0, strlen($stxt) - 1);
                
                $res = $db->query('SELECT * FROM Data WHERE word LIKE ' . $db->quote($stxt . '%'));  
                $res = $res->fetchAll(PDO::FETCH_ASSOC);
                    
                if(count($res) == 0 && strlen($stxt) > 2) // échec > on raccourcit à -2
                    {
                    $stxt = substr($stxt, 0, strlen($stxt) - 1);
                    $res = $db->query('SELECT * FROM Data WHERE word LIKE ' . $db->quote($stxt . '%'));  
                    $res = $res->fetchAll(PDO::FETCH_ASSOC);
                    }
                    
                if(count($res) > 0 ) // Succès après raccourcissement !
                    {
                    
                    $success = 2;
                    
                    // Message : résultats approchants
                    echo '<hr /><p class="warn"><b>', $sqlng_str[63], ' "', $stxt, '"</b> (', count($res), ') :</p>';
                    
                    // On affiche les résultats 
                    foreach ($res as $r) 
                        {
                        echo getHtmOfWordData($r, $db, $lstrel, $fdic, $phpself, $dom, $stxt,
                                            $srl = $sqlng_conf['hilight'], $lg, $htmplug);
                        }
                    }
                }
            }
        else // --- Echec de la recherche dans les notices
            {
            
            // Message : "pas de correspondance pour xxx dans les notices"
            echo '<p class="warn">', $sqlng_str[13], $stxt, $sqlng_str[56], '</p>';
            
            }
        }
    // -------------------------------------------------------
    else // --- Succès de la recherche initiale               
    // -------------------------------------------------------
        {
            $success = 1;
            
            // Message, avec bascule, "a été"/"ont été trouvées"
            $n = ($nbtot_res > 1 ) ? 15 : 16; 
            echo '<p> <b>', $nbtot_res, '</b> ', $sqlng_str[$n];  
            
            // Message "(affichées)"
            if ($nb_res != $nbtot_res)
                {
                echo ' (<b>', $nb_res, '</b> ', $sqlng_str[67], ") </p>\n";
                }
            else
                {
                echo "</p>\n";
                }
            
            // On affiche les notices des résultats       
            foreach ($res as $r) 
                {
                echo getHtmOfWordData($r, $db, $lstrel, $fdic, $phpself, $dom, $stxt,
                                    $srl = $sqlng_conf['hilight'], $lg, $htmplug);
                }
            }
    
    return($success);
    }




/***************************************************************************//**
 * Retourne l'html des propriétés du dico passé en paramètre
 *  
 * @param string $fdic Nom du fichier dico concerné 
 * @return string Html des propriétés du dico
 */
function getHtmOfProperties($fdic)
    {
    
    global $sqlng_conf; // Array des paramètres de configuration
    global $sqlng_str;  // Array des chaînes de langue
    
    // Connexion  à la base-dico sqlite
    $db = new PDO('sqlite:' . $sqlng_conf['scriptdir'] . 'dic/' . $fdic, '', '');        
    
    // extraire le tableau des propriétés
    $res = $db->query('SELECT * FROM Properties');   
    $res = $res->fetch(PDO::FETCH_ASSOC);   
    
    $db = null;
    
    
    // --------------------------------------------------------------------
    // --- Extraction des icônes de langue et affichage du pseudo-titre    
    // --------------------------------------------------------------------
    
    if (function_exists("gd_info"))
        {
        // La bibliothèque GD est disponible
        // > on extraie et affiche les icônes
        $t1 = '<p class="ttr"><img src="' . $sqlng_conf['scriptdir'] . 'lib/geticone.php5?fdic=' . $fdic . '&amp;id=img1" alt="" /> ' . $sqlng_str[34] . ' <img src="' . $sqlng_conf['scriptdir'] . 'lib/geticone.php5?fdic=' . $fdic . '&amp;id=img2" alt="" /></p><hr />';
        }
    else
        {
        // La bibliothèqueGD n'est pas disponible
        // > on affiche des icônes transparentes (pour préserver la mise en page)
        $t1 = '<p class="ttr"><img src="' . $sqlng_conf['scriptdir'] . 'lib/img/rien.gif" alt="" /> ' . $sqlng_str[34] . ' <img src="' . $sqlng_conf['scriptdir'] . 'lib/img/rien.gif" alt="" /></p><hr />';        
        }
    
    
    // ------------------------------
    // --- Nom du fichier            
    // ------------------------------
    
    
    // Le nom du fichier et le lien de téléchargement ne sont affichés que si...
    // 1. il n'y a pas de fichier '.htacces' dans le répertoire des dicos
    // 2. les champs 'protec1' et 'protec2' du dico sont vides
    
    if (!file_exists('../../sqlinguae/dic/.htaccess'))
        {
        if ( (isset($res['protec1']) && $res['protec1'] != '') || (isset($res['protec2']) && $res['protec2'] != '') )
            {
            $t1 .= '<p><b>Fichier sqlite3</b> : <a href="../../sqlinguae/dic/' . $fdic . '">' . $fdic . '  ('
            . filesize('../../sqlinguae/dic/' . $fdic )/1024 . ' Ko)</a></p>';
            }
        }
    
    
    // --------------------------------------
    // --- Mise en forme des propriétés      
    // --------------------------------------
    
    $t = ''; // initialiser l'html
    
    
    if (isset($res['dicName']))
        {
        $t .= '<p><b>' . $sqlng_str[35] . '</b> ';
        if (trim($res['dicName']))
            {
            $t .= htmlspecialchars($res['dicName']) . '</p>';
            }
        else
            {
            $t .= '<span class="grey">' . $sqlng_str[65] . '</span></p>'; // "non défini"
            }
        }
    
    
    // Lang1 : -Name + -Iso
    $t .= '<p><b>' . $sqlng_str[36] . '</b> ';
    $tt = '';
    if (isset($res['langName1']))
        {
        $tt .= htmlspecialchars($res['langName1']) . ' ';
        }
    if (isset($res['langIso1']) && $res['langIso1'])
        {
        $tt .= '(Iso ' . htmlspecialchars($res['langIso1']) . ') ';
        }
    $t .= (strlen($tt) > 1) ? $tt . '</p>' : '<span class="grey">' . $sqlng_str[65] . '</span></p>';


    // Lang2 : -Name + -Iso
    $t .= '<p><b>' . $sqlng_str[37] . '</b> ';
    $tt = '';
    if (isset($res['langName2']))
        {
        $tt .= htmlspecialchars($res['langName2']) . ' ';
        }
    if (isset($res['langIso2']) && $res['langIso2'])
        {
        $tt .= '(Iso ' . htmlspecialchars($res['langIso2']) . ') ';
        }
    $t .= (strlen($tt) > 1) ? $tt . '</p>' : '<span class="grey">' . $sqlng_str[65] . '</span></p>';


    // LangUser : -Name + -Iso
    $t .= '<p><b>' . $sqlng_str[38] . '</b> ';
    $tt = '';
    if (isset($res['langNameUser']))
        {
        $tt .= htmlspecialchars($res['langNameUser']) . ' ';
        }
    if (isset($res['langIsoUser']) && $res['langIsoUser'])
        {
        $tt .= '(Iso ' . htmlspecialchars($res['langIsoUser']) . ') ';
        }
    $t .= (strlen($tt) > 1) ? $tt . '</p>' : '<span class="grey">' . $sqlng_str[65] . '</span></p>';    

    
    // ---
    
    if (isset($res['wordcount'])) $t .= '<p><b>' . $sqlng_str[39] . '</b> ' . number_format(intval($res['wordcount']), 0, ',', ' ') . '</p>';
    
    
    if (isset($res['mainAuthors']))
        {
        $t .= '<p><b>' . $sqlng_str[40] . '</b> ';
        if ($res['mainAuthors'])
            {
            $t .= str_replace('","', '"<br />"', htmlspecialchars($res['mainAuthors'], ENT_NOQUOTES)) . '</p>';
            }
        else
            {
            $t .= '<span class="grey">' . $sqlng_str[65] . '</span></p>'; // "non défini"
            }
        }
    
    
    if (isset($res['altAuthors']))
        {
        $t .= '<p><b>' . $sqlng_str[41] . '</b> ';
        if ($res['altAuthors'])
            {
            $t .= str_replace('","', '"<br />"', htmlspecialchars($res['altAuthors'], ENT_NOQUOTES)) . '</p>';
            }
        else
            {
            $t .= '<span class="grey">' . $sqlng_str[65] . '</span></p>'; // "non défini"
            }
        }
    
    
    if (isset($res['contactAuthor']))
        {
        $t .= '<p><b>' . $sqlng_str[42] . '</b> ';
        if (trim($res['contactAuthor']))
            {
            $t .= htmlspecialchars($res['contactAuthor']) . '</p>';
            }
        else
            {
            $t .= '<span class="grey">' . $sqlng_str[65] . '</span></p>'; // "non défini"
            }
        }
    
    
    if (isset($res['dicStatus']))
        {
        $t .= '<p><b>' . $sqlng_str[43] . '</b> ';
        if (trim($res['dicStatus']))
            {
            $tt = str_replace('<br>', '%%br%%', $res['dicStatus']);
            $tt = htmlspecialchars($tt);
            $tt = str_replace('%%br%%', '<br />', $tt);
            $t .= $tt . '</p>';
            }
        else
            {
            $t .= '<span class="grey">' . $sqlng_str[65] . '</span></p>'; // "non défini"
            }
        }
    
    
    if (isset($res['copyright']))
        {
        $t .= '<p><b>' . $sqlng_str[44] . '</b> ';
        if (trim($res['copyright']))
            {
            $t .= htmlspecialchars($res['copyright']) . '</p>';
            }
        else
            {
            $t .= '<span class="grey">' . $sqlng_str[65] . '</span></p>'; // "non défini"
            }
        }
    
    
    if (isset($res['creationDate']))
        {
        $t .= '<p><b>' . $sqlng_str[45] . '</b> ';
        if (trim($res['creationDate']))
            {
            $t .= htmlspecialchars($res['creationDate']) . '</p>';
            }
        else
            {
            $t .= '<span class="grey">' . $sqlng_str[65] . '</span></p>'; // "non défini"
            }
        }
    
    
    if (isset($res['versionDate']))
        {
        $t .= '<p><b>' . $sqlng_str[46] . '</b> ';
        if (trim($res['versionDate']))
            {
            $t .= htmlspecialchars($res['versionDate']) . '</p>';
            }
        else
            {
            $t .= '<span class="grey">' . $sqlng_str[65] . '</span></p>'; // "non défini"
            }
        }
    
    
    if (isset($res['dicVersionNumber']))
        {
        $t .= '<p><b>' . $sqlng_str[47] . '</b> ';
        if (trim($res['dicVersionNumber']))
            {
            $t .= htmlspecialchars($res['dicVersionNumber']) . '</p>';
            }
        else
            {
            $t .= '<span class="grey">' . $sqlng_str[65] . '</span></p>'; // "non défini"
            }
        }
        

    if (isset($res['dicInfo']))
        {
        $t .= '<p><b>' . $sqlng_str[48] . '</b> ';
        if (trim($res['dicInfo']))
            {
            $tt = str_replace('<br>', '%%br%%', $res['dicInfo']);
            $tt = htmlspecialchars($tt);
            $tt = str_replace('%%br%%', '<br />', $tt);
            $t .= $tt . '</p>';
            }
        else
            {
            $t .= '<span class="grey">' . $sqlng_str[65] . '</span></p>'; // "non défini"
            }
        }
    
    
    if (isset($res['biblio']))
        {
        $t .= '<p><b>' . $sqlng_str[49] . '</b> ';
        if ($res['biblio'])
            {
            $t .= str_replace('","', '"<br />"', htmlspecialchars($res['biblio'], ENT_NOQUOTES)) . '</p>';
            }
        else
            {
            $t .= '<span class="grey">' . $sqlng_str[65] . '</span></p>'; // "non défini"
            }
        }
    
    
    
    
    
    // --- Sécurité
    $t = str_ireplace('<script', '', $t);
    $t = str_ireplace('<noscript', '', $t);
    $t = str_ireplace('<meta', '', $t);
    $t = str_ireplace('document.location', '', $t);
    $t = str_ireplace('<?php', '', $t);
    $t = str_ireplace('src=', '', $t);
    
    $t = str_replace('@', '<span><img src="' . $sqlng_conf['scriptdir'] . 'lib/img/iter.gif" alt="" /></span>', $t);
    
    return($t1 . $t);
    
    }



/***************************************************************************//**
 * Construit et retourne l'html des liens-images vers les greffons (plugins) du
 * dico concerné
 * 
 * @param Object $db Objet PDO d'accès à la base sqlite du dico concerné
 * @return string Code html des liens-images 
 */
function getHtmOfBtPlugins($db)
    {
    
    global $sqlng_conf; // Array des paramètres de configuration
    
    $htm = '';
    
    // Extraire du dico la liste des greffons à afficher
    $res = $db->query('SELECT usePlugins FROM Properties');   
            
    if ($res)
        {
        $res = $res->fetch(PDO::FETCH_ASSOC);
        $res = $res['usePlugins'];
        
        if ($res)
            {
            $res = htmlspecialchars($res);
            $plugs = explode(',', $res);
            }
        else
            {
            return '';
            }
        }
    
    
    foreach ($plugs as $plug)
        {
        $plug = trim($plug, ' "');
        $plug = str_replace('&quot;', '', $plug);
        
        $plug = explode(':', $plug);
        
        if ($plug[0] == 'sqlinguae')
            {
            $plug = $plug[1];
            
            $plugdir = $sqlng_conf['scriptdir'] . 'plugin/' . $plug;
                        
            if (file_exists($plugdir))
                {
                // Vérifier si une icône est fournie
                // nb : on protège le "src=" du nettoyage de sécurité...
                $t = (file_exists( $plugdir . '/ic.gif')) ? '<img !src!"' . $plugdir . '/ic.gif" alt="Plugin:' . $plug . '" title="Plugin:' . $plug . '" />' : '[Plugin:'. $plug .']';
                $htm .= ' <a href="' . $plugdir . '/index.php?wd=[!!!wd!!!]">' . $t . '</a>';
                }
            }
        }
    
    return $htm;
    
    }


    
?>
