<?php


/***************************************************************************//**
 * Mise à jour des statistiques de consultation des dictionnaires
 *  
 * @param string $stxt Motif recherché
 * @param string $fdic Nom du fichier du dictionnaire concerné 
 * @param integer $success (0|1|2) Succès ou non de la recherche
 * @param string $scriptdir Chemin relatif au script  
 */ 
function statUpdate($stxt, $fdic, $success, $scriptdir)
    {
    // A chaque fichier-dico dans /dic/ correspond un fichier de stat dans /stat/
    // son nom reprend le nom du fichier-dico préfixé par 'stat_'. Le fichier de
    // statistiques est automatiquement créé à la première consultation du dico.
    
    
    $db = null;
        
    if ($stxt && $fdic)
        {
        // --- Créer la base des stats du dico si elle n'existe pas
            
        if (!file_exists($scriptdir . 'stat/stat_' . $fdic))
            {
            // Connexion-création de la base-dico sqlite
            $db = new PDO('sqlite:' . $scriptdir . 'stat/stat_' . $fdic, '', '');    
                
            // Créer la table des stats
            $db->exec( 'CREATE TABLE Queries
                                        (
                                        word TEXT NOT NULL, 
                                        nb INTEGER, 
                                        success INTEGER,
                                        date TEXT
                                        )'
                         );                                          
            }
        else
            {
            // La base des stats du dico existe déjà, on s'y connecte
            $db = new PDO('sqlite:' . $scriptdir . 'stat/stat_' . $fdic, '', '');   
            }
            
        // -----------------------------------
        // --- Compléter les statistiques     
        // -----------------------------------
            
        $res = $db->query('SELECT word FROM Queries WHERE word = ' . $db->quote($stxt) );
        $res = $res->fetch(PDO::FETCH_ASSOC);
            
        // Le mot recherché est déjà présent dans la base > on le complète    
        if ( $res['word'] )
            {
                
            $db->exec('UPDATE Queries
                                SET nb = nb + 1,
                                    success = ' . $success . ',
                                    date = ' . $db->quote(date('Y-m-d')) . '
                                WHERE word = ' . $db->quote($stxt) );
                
            }
        // Le mot recherché est absent de la base > on l'ajoute
        else
            {
            $db->exec('INSERT INTO Queries (
                                            word,
                                            nb,
                                            success,
                                            date
                                            ) 
                                    VALUES  (' . $db->quote($stxt) . ',
                                            1 ,
                                            ' . $success . ',
                                            ' . $db->quote(date('Y-m-d')) . '
                                            )'
                    );
                
            }
            
        $db = null;
        }
    }


?>
